@@ -3,10 +3,19 @@ import os
 from copy import copy
 from inspect import isclass
 import shutil
+from operator import itemgetter
+import itertools
 from pathlib import Path
 import logging
 import attrs
-from .utils import splitext, to_mime, subpackages, classproperty
+from .utils import (
+    to_mime,
+    subpackages,
+    classproperty,
+    fspaths_converter,
+    to_mime_format_name,
+    STANDARD_NAMESPACES,
+)
 from .exceptions import FileFormatsError, FormatMismatchError, FormatConversionError
 
 
@@ -19,15 +28,9 @@ CHECK_ANNOTATION = "__fileformats_check__"
 logger = logging.getLogger("fileformats")
 
 
-def fspaths_converter(fspaths):
-    """Ensures fs-paths are a set of pathlib.Path"""
-    if isinstance(fspaths, (str, Path, bytes)):
-        fspaths = [fspaths]
-    return set((Path(p) if isinstance(p, str) else p).absolute() for p in fspaths)
-
-
 @attrs.define
 class Metadata:
+    """Manually set and lazily loaded metadata of a FileSet"""
 
     loaded: dict = attrs.field(factory=dict, converter=dict)
     _fileset = attrs.field(default=None, init=False, repr=False)
@@ -47,11 +50,12 @@ class Metadata:
         if hasattr(self._fileset, "load_metadata"):
             loaded_dict = self._fileset.load_metadata()
             if not overwrite:
-                if mismatching := [
+                mismatching = [
                     k
                     for k in set(self.loaded) & set(loaded_dict)
                     if self.loaded[k] != loaded_dict[k]
-                ]:
+                ]
+                if mismatching:
                     raise FileFormatsError(
                         "Mismatch in values between loaded and loaded metadata values, "
                         "use 'load(overwrite=True)' to overwrite:\n"
@@ -84,11 +88,13 @@ class FileSet:
     """
 
     fspaths: set[Path] = attrs.field(default=None, converter=fspaths_converter)
-    metadata: Metadata = attrs.field(factory=dict, converter=Metadata, kw_only=True)
+    metadata: Metadata = attrs.field(
+        factory=dict, converter=Metadata, repr=False, kw_only=True
+    )
 
-    # Explicitly set the Internet Assigned Numbers Authority (https://iana.org) MIME
+    # Explicitly set the Internet Assigned Numbers Authority (https://iana_mime.org) MIME
     # type to None for any base classes that should not correspond to a MIME type.
-    iana = None
+    iana_mime = None
 
     # Store converters registered by @converter decorator that convert to FileSet
     # NB: each class will have its own version of this dictionary
@@ -102,7 +108,10 @@ class FileSet:
 
     @fspaths.validator
     def validate_fspaths(self, _, fspaths):
-        if missing := [p for p in fspaths if not p or not p.exists()]:
+        if not fspaths:
+            raise FileFormatsError(f"No file-system paths provided to {self}")
+        missing = [p for p in fspaths if not p or not p.exists()]
+        if missing:
             missing_str = "\n".join(str(p) for p in missing)
             all_str = "\n".join(str(p) for p in fspaths)
             msg = (
@@ -189,7 +198,10 @@ class FileSet:
         iter(str)
             an iterator over all properties names marked as "required"
         """
+        fileset_props = dir(FileSet)
         for attr_name in dir(cls):
+            if attr_name in fileset_props:
+                continue
             klass_attr = getattr(cls, attr_name)
             if isinstance(klass_attr, property):
                 try:
@@ -199,6 +211,19 @@ class FileSet:
                 else:
                     yield attr_name
 
+    def required_paths(self):
+        """Returns all fspaths that are required for the format"""
+        required = set()
+        for prop_name in self.required_properties:
+            prop = getattr(self, prop_name)
+            if prop in self.fspaths:
+                required.add(prop)
+
+    def trim_paths(self):
+        """Trims paths in fspaths to only those that are "required" by the format class
+        i.e. returned by a required property"""
+        self.fspaths = self.required_paths()
+
     @classmethod
     def checks(cls):
         """Find all methods used to check the validity of the file format
@@ -209,7 +234,10 @@ class FileSet:
             an iterator over all method names marked as a "check"
         """
         # Loop through all attributes and find methods marked by CHECK_ANNOTATION
+        fileset_props = dir(FileSet)
         for attr_name in dir(cls):
+            if attr_name in fileset_props:
+                continue
             klass_attr = getattr(cls, attr_name)
             try:
                 klass_attr.__annotations__[CHECK_ANNOTATION]
@@ -218,7 +246,9 @@ class FileSet:
             else:
                 yield attr_name
 
-    def copy(self, dest_dir: Path, stem: str = None, symlink: bool = False):
+    def copy_to(
+        self, dest_dir: Path, stem: str = None, symlink: bool = False, trim: bool = True
+    ):
         """Copies the file-set to a new directory, optionally renaming the files
         to have consistent name-stems.
 
@@ -231,6 +261,8 @@ class FileSet:
             directory, by default the original file name is used
         symlink : bool, optional
             Use symbolic links instead of copying files to new location, false by default
+        trim : bool, optional
+            Only copy the paths in the file-set that are "required" by the format, true by default
         """
         dest_dir = Path(dest_dir)  # ensure a Path not a string
         if symlink:
@@ -239,7 +271,11 @@ class FileSet:
             copy_dir = shutil.copytree
             copy_file = shutil.copyfile
         new_paths = []
-        for fspath in self.fspaths:
+        if trim:
+            fspaths_to_copy = self.required_paths()
+        else:
+            fspaths_to_copy = self.fspaths
+        for fspath in fspaths_to_copy:
             new_fname = stem + ".".join(fspath.suffixes) if stem else fspath.name
             new_path = dest_dir / new_fname
             if fspath.is_dir():
@@ -249,26 +285,27 @@ class FileSet:
             new_paths.append(new_path)
         return type(self)(new_paths)
 
-    def select_by_ext(self, fileformat: type = None) -> Path:
+    def select_by_ext(self, fileformat: type = None, allow_none: bool = False) -> Path:
         """Selects a single path from a set of file-system paths based on the file
         extension
 
         Parameters
         ----------
-        fspaths : set[Path]
-            the file-system paths to select from
-        ext : str
-            the file extension to select
+        fileformat : type
+            the format class of the path to select
+        allow_none : bool
+            whether to return None instead of raising an error if the file is not found
 
         Returns
         -------
-        Path
-            the selected file-system path that matches the extension
+        Path or None
+            the selected file-system path that matches the extension or None if not found
+            and `allow_none` is True
 
         Raises
         ------
         FileFormatError
-            if no paths match the extension
+            if no paths match the extension and `allow_none` is False
         FileFormatError
             if more than one paths matches the extension
         """
@@ -281,6 +318,8 @@ class FileSet:
             pass
         matches = self.matching_exts(self.fspaths, exts)
         if not matches:
+            if allow_none:
+                return None
             paths_str = ", ".join(str(p) for p in self.fspaths)
             raise FormatMismatchError(
                 f"No matching files with extensions in {exts} in file set {paths_str}"
@@ -341,9 +380,7 @@ class FileSet:
         task = cls.get_converter(source_format=type(fileset), name=task_name, **kwargs)
         result = task(in_file=fileset, plugin=plugin)
         out_file = result.output.out_file
-        if isinstance(out_file, (str, bytes, os.PathLike)):
-            out_file = cls.from_primary(out_file)
-        elif not isinstance(out_file, cls):
+        if not isinstance(out_file, cls):
             out_file = cls(out_file)
         return out_file
 
@@ -356,7 +393,7 @@ class FileSet:
         ----------
         source_format : type
             the format to convert from
-        task_name : str
+        name : str
             the name given to the converter task
         **kwargs
             passed on to the task init method to customise the conversion
@@ -369,9 +406,9 @@ class FileSet:
         Raises
         ------
         FileFormatConversionError
-            _description_
+            no converters found between source and dest format
         FileFormatConversionError
-            _description_
+            ambiguous (i.e. more than one) converters found between source and dest format
         """
         converter_tuple = None
         # Only access converters to the specific class, not superclasses (which may not
@@ -406,75 +443,66 @@ class FileSet:
         if kwargs:
             conv_kwargs = copy(conv_kwargs)
             conv_kwargs.update(kwargs)
-        # Make recognisable default task name
-        # if task_name is None:
-        #     task_name = f"{source_format.__name__}_to_{cls.__name__}"
         return converter(name=name, **conv_kwargs)
 
-    @classmethod
-    def include_adjacents(
-        cls,
-        fspaths: set[Path],
-        duplicate_ext: bool = False,
-        multipart_ext: bool = True,
-    ) -> set[Path]:
-        """Adds any "adjacent files", i.e. any files with the same stem but different
-        extension, if that suffix isn't already present in the existing fspaths
-
-        Parameters
-        ----------
-        fspaths : list[Path]
-            the paths to find the suffies
-        duplicate_ext : bool, optional
-            whether to include adjacent files if there is already a file with the same
-            extension in the file set
-        multipart_ext : bool, optional
-            whether to treat paths with multiple "." as having one long suffix,
-            e.g. "image.nii.gz"
-        """
-        # Create a copy of the fspaths provided
-        fspaths = set(fspaths)
-
-        for fspath in list(fspaths):
-            stem = splitext(fspath, multi=multipart_ext)[0]
-            for neighbour in fspath.parent.iterdir():
-                neigh_stem, neigh_ext = splitext(neighbour, multi=multipart_ext)
-                if neigh_stem == stem:
-                    if duplicate_ext or neigh_ext not in (
-                        splitext(p, multi=multipart_ext)[-1] for p in fspaths
-                    ):
-                        fspaths.add(neighbour)
-        return fspaths
-
-    @classmethod
-    def from_primary(
-        cls,
-        fspaths: set[Path],
-        **kwargs,
-    ):
-        """Factory method to create a file-set object with all the secondary files
-        that inferred/referenced from the primary fspath included in the file-set paths.
-
-        Parameters
-        ----------
-        fspaths : list[Path]
-            the paths to find the suffies
-        duplicate_ext : bool, optional
-            whether to include adjacent files if there is already a file with the same
-            extension in the file set
-        multipart_ext : bool, optional
-            whether to treat paths with multiple "." as having one long suffix,
-            e.g. "image.nii.gz"
-        """
-        fspaths = fspaths_converter(fspaths)
-        return cls(cls.include_adjacents(fspaths=fspaths, **kwargs))
+    @classproperty
+    def namespace(cls):
+        """The "namespace" the format belongs to under the "fileformats" umbrella
+        namespace"""
+        module_parts = cls.__module__.split(".")
+        if module_parts[0] != "fileformats":
+            raise FileFormatsError(
+                f"Cannot create reversible MIME type for {cls} as it is not in the "
+                "fileformats namespace"
+            )
+        return module_parts[1]
 
     @classproperty
     def all_formats(cls):
+        """Iterate over all formats in fileformats.* namespaces"""
         if cls._all_formats is None:
             cls._all_formats = [
-                f for f in FileSet.subclasses() if f.__dict__.get("iana", True)
+                f for f in FileSet.subclasses() if f.__dict__.get("iana_mime", True)
             ]
         return cls._all_formats
 
+    @classproperty
+    def standard_formats(cls):
+        """Iterate over all formats in the standard fileformats.* namespaces"""
+        return (f for f in cls.all_formats if f.namespace in STANDARD_NAMESPACES)
+
+    @classproperty
+    def formats_by_iana_mime(cls):
+        """a dictionary containing all formats by their IANA MIME type (if applicable)"""
+        if cls._formats_by_iana_mime is None:
+            cls._formats_by_iana_mime = {
+                f.iana_mime: f
+                for f in FileSet.all_formats
+                if getattr(f, "iana_mime", None) is not None
+            }
+        return cls._formats_by_iana_mime
+
+    @classproperty
+    def formats_by_name(cls):
+        """a dictionary containing lists of formats by their translated class names,
+        i.e. their can be more than one format with the same translated name"""
+        if cls._formats_by_name is None:
+            cls._formats_by_name = {
+                k: set(v for _, v in g)
+                for k, g in itertools.groupby(
+                    sorted(
+                        (
+                            (to_mime_format_name(f.__name__), f)
+                            for f in FileSet.all_formats
+                            if getattr(f, "iana_mime", None) is None
+                        ),
+                        key=itemgetter(0),
+                    ),
+                    key=itemgetter(0),
+                )
+            }
+        return cls._formats_by_name
+
     _all_formats = None
+    _formats_by_iana_mime = None
+    _formats_by_name = None
