@@ -1,13 +1,20 @@
 from warnings import warn
+from pathlib import Path
 import json
-import yaml
-from ..core import File, __version__, mark
+from ..core import __version__, mark
+from ..generic import File
+from ..core.utils import MissingExtendedDependency, import_converters
+
+try:
+    import yaml
+except ImportError:
+    yaml = MissingExtendedDependency("yaml", __name__)
 
 
-class Serialization(File):
+class DataSerialization(File):
     "Base class for text-based hierarchical data-serialization formats, e.g. JSON, YAML"
 
-    iana = None
+    iana_mime = None
 
     @mark.check
     def load(self):
@@ -15,50 +22,46 @@ class Serialization(File):
         raise NotImplementedError
 
     @classmethod
-    def save_new(dct, fspath):
+    def save_new(fspath: Path, dct: dict):
         """Serialise a dictionary to a new file"""
         raise NotImplementedError
 
 
-class Xml(Serialization):
+class Xml(DataSerialization):
     ext = ".xml"
 
 
-class Json(Serialization):
+class Json(DataSerialization):
     ext = ".json"
 
+    @mark.check
     def load(self):
         with open(self.fspath) as f:
             dct = json.load(f)
         return dct
 
     @classmethod
-    def save_new(cls, dct, fspath):
+    def save_new(cls, fspath, dct):
         with open(fspath, "w") as f:
             json.dump(dct, f)
         return cls(fspath)
 
 
-class Yaml(Serialization):
+class Yaml(DataSerialization):
     ext = ".yaml"
     alternate_exts = (".yml",)
 
+    @mark.check
     def load(self):
         with open(self.fspath) as f:
             dct = yaml.load(f, Loader=yaml.Loader)
         return dct
 
     @classmethod
-    def save_new(cls, dct, fspath):
+    def save_new(cls, fspath, dct):
         with open(fspath, "w") as f:
             yaml.dump(dct, f)
         return cls(fspath)
 
 
-try:
-    from .converters import *
-except ImportError:
-    warn(
-        f"could not import converters for {__name__}  module, please install "
-        "fileformats[converters] if conversion is desired"
-    )
+import_converters(__name__)
